/**
 * @file /include/ecl/devices/detail/socket_error_handler_pos.hpp
 *
 * @brief Defines error handlers for posix sockets.
 *
 * @date September 2009
 **/
/*****************************************************************************
** Ifdefs
*****************************************************************************/

#ifndef ECL_DEVICES_SOCKET_ERROR_HANDLER_POS_HPP_
#define ECL_DEVICES_SOCKET_ERROR_HANDLER_POS_HPP_

/*****************************************************************************
** Cross platform
*****************************************************************************/

#include <ecl/config/ecl.hpp>
#ifndef ECL_IS_MAC
#ifdef ECL_IS_POSIX

/*****************************************************************************
** Includes
*****************************************************************************/

#include <string>
#include <ecl/errors/handlers.hpp>

/*****************************************************************************
** Namespaces
*****************************************************************************/

namespace ecl {
namespace devices {

/*****************************************************************************
** Methods
*****************************************************************************/
/**
 * This function generates a custom ecl::Error response
 * for posix error numbers generated by posix <i>socket</i> calls.
 * @return Error : the error type.
 */
ecl::Error socket_error();
/**
 * This function generates a custom ecl::Error response
 * for posix error numbers generated by posix <i>bind</i> calls.
 * @return Error : the error type.
 */
ecl::Error bind_error();
/**
 * This function generates a custom ecl::Error response
 * for posix error numbers generated by posix <i>accept</i> calls.
 * @return Error : the error type.
 */
ecl::Error accept_error();
/**
 * This function generates a custom ecl::Error response
 * for posix error numbers generated by posix <i>recv</i> calls.
 * @return Error : the error type.
 */
ecl::Error receive_error();
/**
 * This function generates a custom ecl::Error response
 * for posix error numbers generated by posix <i>ioctl</i> calls.
 * @return Error : the error type.
 */
ecl::Error ioctl_error();
/**
 * This function generates a custom ecl::Error response
 * for posix error numbers generated by posix <i>send</i> calls.
 * @return Error : the error type.
 */
ecl::Error send_error();
/**
 * This function generates a custom ecl::Error response
 * for posix error numbers generated by posix <i>gethostbyname</i> calls.
 * @return Error : the error type.
 */
ecl::Error gethostbyname_error();
/**
 * This function generates a custom ecl::Error response
 * for posix error numbers generated by posix <i>connect</i> calls.
 * @return Error : the error type.
 */
ecl::Error connection_error();

} // namespace devices
} // namespace ecl

#endif /* ECL_IS_POSIX */
#endif /* !ECL_IS_MAC */
#endif /* ECL_DEVICES_SOCKET_ERROR_HANDLER_POS_HPP_ */
